<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 12.09.18
 * Time: 17:18
 */

namespace WS\EMarketing;


use WS\EMarketing\Exception\APIRequestException;
use WS\EMarketing\Exception\RequestValidationException;
use WS\EMarketing\Request\AbstractRequest;
use WS\EMarketing\Response\AbstractResponse;

class API
{
    /**
     * @var string
     */
    protected $_apiToken;

    /**
     * API constructor.
     * @param string $apiToken
     */
    public function __construct($apiToken)
    {
        $this->_apiToken = $apiToken;
    }

    /**
     * @param AbstractRequest $request
     * @return AbstractResponse
     * @throws RequestValidationException
     * @throws APIRequestException
     * @throws Exception\ResponseException
     */
    public function doRequest(AbstractRequest $request)
    {
        if (!$request->isValid($error)) {
            throw new RequestValidationException($error);
        }

        $ch = curl_init();
        $endpoint = $request->getEndpoint();
        if (defined('EMARKETING_TEST') && EMARKETING_TEST === true) {
            $endpoint = str_replace('app.', 'sandbox.', $endpoint);
        }
        if (!$request->isPost()) {
            $endpoint .= '?' . $request->buildQuery();
        }
        curl_setopt($ch, CURLOPT_URL, $endpoint);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: ' . $this->_apiToken,
            'Content-Type: application/json; charset=utf-8',
            'X-Platform: jst_emarketing/JTL-Shop ' . JTL_VERSION,
        ]);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        if ($request->isPost()) {
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($request));
        }
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $request->getMethod());

        $response = curl_exec($ch);
        if ($response === false) {
            throw new APIRequestException("cURL Error: " . curl_error($ch), curl_errno($ch));
        }

        $statusCode = (int)curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($statusCode < 200 || $statusCode >= 300) {
            throw new APIRequestException("Unexpected Status Code: {$statusCode}, " . print_r($response, 1));
        }

        return $request->getResponseObject()->fromResponse($response);

    }

}