<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 24.09.18
 * Time: 10:38
 */

namespace WS\EMarketing\Request;


use WS\EMarketing\Response\Success;

class Settings extends AbstractRequest
{

    const ENDPOINT = 'https://app.emarketing.com/%s/settings';
    const METHOD = self::METHOD_PUT;
    /**
     * Used to retrieve categories
     * Regex: ^(https?):\/\/[^\s\/$.?#].[^\s]*$
     *
     * @var string
     */
    public $categoriesApiEndpoint;
    /**
     * Used to retrieve products
     * Regex: ^(https?):\/\/[^\s\/$.?#].[^\s]*$
     *
     * @var string
     */
    public $productsApiEndpoint;
    /**
     * The ID of the root category
     *
     * @var string
     */
    public $shopCategoryRootId = "0";
    /**
     * Used to update the AdWords conversion tracker
     * Regex: ^(https?):\/\/[^\s\/$.?#].[^\s]*$
     *
     * @var null|string
     */
    public $conversionTrackerApiEndpoint;
    /**
     * Used to update the AdWords lead tracker
     * Regex: ^(https?):\/\/[^\s\/$.?#].[^\s]*$
     *
     * @var null|string
     */
    public $leadTrackerApiEndpoint;

    /**
     * Used to update the price of a single product pattern
     * Regex: ^(https?):\/\/[^\s/$.?#].[^\s]*$
     *
     * @var string
     */
    public $singleProductApiEndpoint;

    /**
     * Used to update the Facebook pixel pattern
     * Regex: ^(https?):\/\/[^\s/$.?#].[^\s]*$
     * @var null|string
     */
    public $facebookPixelApiEndpoint;

    protected $_target;

    public function __construct($target)
    {
        $this->_target = $target;
    }

    public function getEndpoint()
    {
        return sprintf(self::ENDPOINT, $this->_target);
    }

    /**
     * Checks if request is valid
     *
     * @return bool
     */
    public function isValid(&$errorMessage)
    {
        if($this->_target === 'datafeed_api'){
            if (!$this->categoriesApiEndpoint || !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->categoriesApiEndpoint)) {
                $errorMessage = "Categories endpoint must be a valid URL!";
                return false;
            }
            if (!$this->productsApiEndpoint || !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->productsApiEndpoint)) {
                $errorMessage = "Products endpoint must be a valid URL!";
                return false;
            }

            if ($this->conversionTrackerApiEndpoint && !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->conversionTrackerApiEndpoint)) {
                $errorMessage = "Conversion Tracker endpoint must be a valid URL!";
                return false;
            }
            if ($this->leadTrackerApiEndpoint && !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->leadTrackerApiEndpoint)) {
                $errorMessage = "Lead Tracker endpoint must be a valid URL!";
                return false;
            }
            if ($this->facebookPixelApiEndpoint && !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->facebookPixelApiEndpoint)) {
                $errorMessage = "Facebook Pixel endpoint must be a valid URL!";
                return false;
            }

            if ($this->shopCategoryRootId === '') {
                $errorMessage = "Category Root ID can't be empty!";
                return false;
            }
        }elseif($this->_target === 'repricing_api'){
            $this->shopCategoryRootId = null;

            if (!$this->singleProductApiEndpoint || !preg_match('/^(https?):\/\/[^\s\/$.?#].[^\s]*$/', $this->singleProductApiEndpoint)) {
                $errorMessage = "SingleProduct endpoint must be a valid URL!";
                return false;
            }
        }

        return true;
    }

    /**
     * @return \WS\EMarketing\Response\AbstractResponse
     */
    public function getResponseObject()
    {
        return new Success();
    }
}