<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 13.09.18
 * Time: 10:27
 */

namespace WS\EMarketing\Shop\Request;

/**
 * Class AbstractRequest
 * @package WS\EMarketing\Shop\Request
 */
abstract class AbstractRequest
{
    const PARAM_LIMIT = 'limit';
    const PARAM_OFFSET = 'offset';
    const PARAM_LIMITVARIANTS = 'limitvariants';
    const PARAM_ID = 'id';
    const PARAM_TYPE = 'type';
    const PARAM_TAG = 'tag';
    const PARAM_SNIPPET = 'snippet';
    const PARAM_OFFER_ID = 'offer_id';
    const PARAM_CURRENCY = 'currency';
    const PARAM_PRICE = 'price';
    const PARAM_GTIN = 'gtin';
    const PARAM_TARGET_COUNTRY = 'target_country';
    const PARAM_ADD_TO_CART_SNIPPET = 'add_to_cart_snippet';
    const PARAM_GLOBAL_TAG = 'global_tag';
    const PARAM_PURCHASE_SNIPPET = 'purchase_snippet';
    const PARAM_VIEW_CONTENT_SNIPPET = 'view_content_snippet';

    /**
     * @var array
     */
    protected $_data = [];

    /**
     * AbstractRequest constructor.
     * @param array $data
     */
    public function __construct($data)
    {
        foreach ($data as $key => $value) {
            switch ($key) {
                case self::PARAM_OFFSET:
                case self::PARAM_LIMITVARIANTS:
                case self::PARAM_LIMIT:
                    $this->_data[strtolower($key)] = (int)filter_var($value, FILTER_SANITIZE_NUMBER_INT);
                    break;
                case self::PARAM_ID:
                case self::PARAM_TYPE:
                case self::PARAM_CURRENCY:
                case self::PARAM_TARGET_COUNTRY:
                case self::PARAM_GTIN:
                case self::PARAM_OFFER_ID:
                    $this->_data[strtolower($key)] = trim(filter_var($value, FILTER_SANITIZE_STRING));
                    break;
                case self::PARAM_TAG:
                case self::PARAM_SNIPPET:
                case self::PARAM_ADD_TO_CART_SNIPPET:
                case self::PARAM_GLOBAL_TAG:
                case self::PARAM_PURCHASE_SNIPPET:
                case self::PARAM_VIEW_CONTENT_SNIPPET:
                    $this->_data[strtolower($key)] = trim($value);
                    break;
                case self::PARAM_PRICE:
                    $this->_data[strtolower($key)] = (float)filter_var($value, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
                default:
                    /* @TODO: LOG EVENT ? */
            }
        }
    }

    /**
     * @param string $shopToken
     * @param string|null $authToken
     * @return bool
     */
    public static function isAuthenticated($shopToken, $authToken = null)
    {
        if ($authToken === null) {
            $authToken = self::getAuthorizationHeader();
        }
        return $shopToken === $authToken;
    }

    /**
     * @return null|string
     */
    protected static function getAuthorizationHeader()
    {
        $headers = null;
        if (isset($_SERVER['Authorization'])) {
            $headers = trim($_SERVER["Authorization"]);
        } else if (isset($_SERVER['HTTP_AUTHORIZATION'])) { //Nginx or fast CGI
            $headers = trim($_SERVER["HTTP_AUTHORIZATION"]);
        } else if (isset($_SERVER['X-Authorization'])) {
            $headers = trim($_SERVER["X-Authorization"]);
        } else if (isset($_SERVER['HTTP_X_AUTHORIZATION'])) { //Nginx or fast CGI
            $headers = trim($_SERVER["HTTP_X_AUTHORIZATION"]);
        } else if (function_exists('apache_request_headers')) {
            $requestHeaders = apache_request_headers();
            // Server-side fix for bug in old Android versions (a nice side-effect of this fix means we don't care about capitalization for Authorization)
            $requestHeaders = array_combine(array_map('ucwords', array_keys($requestHeaders)), array_values($requestHeaders));
            if (isset($requestHeaders['Authorization'])) {
                $headers = trim($requestHeaders['Authorization']);
            } else if (isset($requestHeaders['X-Authorization'])) {
                $headers = trim($requestHeaders['X-Authorization']);
            }
        }
        return $headers;
    }

    /**
     * @param $name
     * @return mixed
     */
    public function __get($name)
    {
        return $this->_data[$name];
    }

    /**
     * @param string $name
     * @param mixed $value
     */
    public function __set($name, $value)
    {
        $this->_data[] = $value;
    }

    public function __isset($name)
    {
        return array_key_exists($name, $this->_data);
    }

    /**
     * @return bool
     */
    abstract public function dataIsValid();
}