<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 13.09.18
 * Time: 10:19
 */

namespace WS\EMarketing\Shop;


use WS\EMarketing\Exception\WrongTypeException;
use WS\EMarketing\Shop\Request\AbstractRequest;
use WS\EMarketing\Shop\Request\Categories;
use WS\EMarketing\Shop\Request\Product;
use WS\EMarketing\Shop\Request\Products;
use WS\EMarketing\Shop\Request\Tracker;

final class RequestFactory
{

    const TYPE_PRODUCT = 'product';
    const TYPE_PRODUCTS = 'products';
    const TYPE_CATEGORIES = 'categories';
    const TYPE_LEADTRACKER = 'lead_tracker';
    const TYPE_CONVERSIONTRACKER = 'conversion_tracker';
    const TYPE_FACEBOOK_PIXEL = 'facebook_pixel';

    /**
     * RequestFactory constructor.
     * @throws \BadMethodCallException
     */
    private function __construct()
    {
        throw new \BadMethodCallException(__CLASS__ . ' can\'t be instantiated.');
    }


    /**
     * @param $type
     * @param $data
     * @return AbstractRequest
     * @throws WrongTypeException
     */
    public static function factory($type, $data)
    {
        switch ($type) {
            case self::TYPE_PRODUCTS:
                return new Products($data);

            case self::TYPE_PRODUCT:
                return new Product($data);

            case self::TYPE_CATEGORIES:
                return new Categories($data);

            case self::TYPE_CONVERSIONTRACKER:
            case self::TYPE_LEADTRACKER:
            case self::TYPE_FACEBOOK_PIXEL:
                $data['type'] = $type;
                return new Tracker($data);

            default:
                throw new WrongTypeException($type);
        }
    }

}
